///
/// @file        adc_imx6.h
/// @copyright   Copyright (c) 2015 Toradex AG
/// $Author: raja.sivasankaran $
/// $Rev: 3073 $
/// $Date: 2015-12-30 15:11:00 +0530 (Wed, 30 Dec 2015) $
/// @brief       Library to support ADC on Toradex modules
/// @test        Colibri and Apalis iMX6
/// @target      Colibri and Apalis iMX6.
///
/// @anchor imx6adc_pins
/// <h2 class ="groupheader">ADC Pin Availability on Colibri iMX6 and Apalis iMX6 modules</h2>
/// The tables below list the module pins, on which ADC functionality is available.\n
/// The Compatible ADC channels as defined on the Colibri and Apalis Evaluation board are listed
/// in the column "ADC".
/// <h3>Colibri iMX6</h3>
/// |   SODIMM   |  GPIO  |  AltFn  |  ADC(Analog input) |
/// | :--------: | :----: | :-----: | :----------------: |
/// |     8      |   -    |     -   |       ADC1 (0)     |
/// |     6      |   -    |     -   |       ADC2 (1)     |
/// |     4      |   -    |     -   |       ADC3 (2)     |
/// |     2      |   -    |     -   |       ADC4 (3)     |
///
/// <h3>Apalis iMX6</h3>             
/// |   MXM3     |  GPIO  |  AltFn  |  ADC(Analog input) | 
/// | :--------: | :----: | :-----: | :----------------: | 
/// |     305    |   -    |     -   |       ADC1 (0)     |
/// |     307    |   -    |     -   |       ADC2 (1)     |
/// |     309    |   -    |     -   |       ADC3 (2)     |
/// |     311    |   -    |     -   |       ADC4 (3)     |
///
///
/// @page    pcfg_adc     ADC Configuration Options
/// @section scfg_adc_imx6 iMX6 ADC Configurations
/// @anchor acfg_adc_imx6
/// For a list of compatible parameters, refer to @ref acfg_adc "Compatible ADC Configurations".
/// <table class="libParam" >
///   <tr><th>                              Parameter Name </th><th> Value   </th><th> Description</th></tr>
///   <tr><td class="paramname"            >ioAdc          </td><td> DWORD   </td><td> GPIO used as ADC pin in uIo format.<br>The @ref imx6adc_pins "Imx6 ADC pins" shows a list of ADC functionality IOs.</td></tr>
///   <tr><td class="paramname"            >BitResolution  </td><td> 1 to 32 </td><td> Defines the ADC resolution in Bits.<br>Currently, only 12 bit resolution is supported.</td></tr>
///   <tr><td class="paramname"            >AvgSamples     </td><td> 1 to 64 </td><td> Defines the number of ADC conversion which will be performed for a single averaged result.<br>All values from 1 to 64 are accepted.</td></tr>
/// </table>
///

#ifndef __ADC_IMX6_H__
#define __ADC_IMX6_H__

#include <windows.h>
#include "TdxCommon.h"

#ifdef __cplusplus
extern "C" {
#endif

//******************************************************************************
/// Initialize HANDLE for the ADC channel.\n
/// This function call does not modify any hardware registers. Therefore the
/// returned handle can be used to make additional configuration settings, which
/// will become active when calling Imx6Adc_Open().
/// @param[in]    portName    ADC channel name. Can be one of these: <ul><li>L"ADC1"</li><li>L"ADC2"</li><li>L"ADC3"</li><li>L"ADC4"</li></ul>
/// @retval       Handle to the ADC port. NULL if the function failed.
HANDLE Imx6Adc_Init(const TCHAR *portName);

//******************************************************************************
/// Finally close the HANDLE. The port must be closed before.
/// @param[in]    hPort       HANDLE to ADC channel
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Adc_Deinit(HANDLE hPort);

//******************************************************************************
/// Apply configuration to the hardware, If it is stored in registry path.
/// @param[in]    hPort       HANDLE to ADC channel
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Adc_Open(HANDLE hPort);

//******************************************************************************
/// Close ADC
/// @param[in]    hPort       HANDLE to ADC channel
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Adc_Close(HANDLE hPort);

//******************************************************************************
/// Read ADC channel
/// @param[in]    hPort       HANDLE to ADC channel
/// @param[out]   microVolts  ADC value in Micro Volts.
/// @param[in]    maxBytes    Ignored, the implementation always assumes 4.
/// @return       Number of Bytes received. 0 means failure.
DWORD Imx6Adc_Read(HANDLE hPort, DWORD *microVolts, DWORD maxBytes);

//******************************************************************************
/// Retrieves a DWORD configuration parameter.
/// @param[in]    hPort       HANDLE to ADC channel
/// @param[in]    paramName   Parameter value to retrieve\n
///                           Refer to the @ref scfg_adc_imx6 for a list of supported parameters.
/// @param[out]   pValue      Data obtained
/// @return                   Number of bytes read. 0 if paramName not found.
DWORD Imx6Adc_GetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD *pValue);

//******************************************************************************
/// Set a DWORD configuration parameter.
/// @param[in]    hPort       HANDLE to ADC channel
/// @param[in]    paramName   Parameter value to configure.\n
///                           Refer to the @ref scfg_adc_imx6 for a list of supported parameters.
/// @param[in]    value       Data to be set
/// @param[in]    storageType StoreToRegistry = parameter will also be stored in the registry\n
///                           StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Adc_SetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD value, ParamStorageType storageType);

//******************************************************************************
/// Get the revision of this Adc_imx6 library
/// @param[out]    ver        library version, consisting of major, minor and build version
void Imx6Adc_GetVersion(tVersion *ver);

#ifdef __cplusplus
}
#endif

#endif